@@ -9,18 +9,25 @@ from dataclasses import dataclass
 from typing import Any, Optional, Sequence, Type, TypeVar, overload
 from urllib.request import urlopen
 
-from beet import BinaryFile, BinaryFileBase, Cache, File, FormattedPipelineException
+from beet import BinaryFile, BinaryFileBase, BubbleException, Cache, File
 from beet.core.utils import FileSystemPath
 
 FileType = TypeVar("FileType", bound=File[Any, Any])
 
 
-class InvalidFragment(FormattedPipelineException):
+class InvalidFragment(BubbleException):
     """Raised when a fragment can not be processed."""
 
+    message: str
+    line: int
+
     def __init__(self, message: str, line: int):
         super().__init__(message, line)
-        self.message = message + f" (line {line + 1})"
+        self.message = message
+        self.line = line
+
+    def __str__(self) -> str:
+        return f"{self.message} (line {self.line + 1})"
 
 
 @dataclass(frozen=True)
@@ -43,11 +50,11 @@ class Fragment:
         ...
 
     @overload
-    def expect(self, name1: str) -> str:
+    def expect(self, name1: str, /) -> str:
         ...
 
     @overload
-    def expect(self, name1: str, name2: str, *names: str) -> Sequence[str]:
+    def expect(self, name1: str, name2: str, /, *names: str) -> Sequence[str]:
         ...
 
     def expect(self, *names: str):
@@ -64,7 +71,15 @@ class Fragment:
             return self.arguments[0]
         return self.arguments
 
-    def as_file(self, file_type: Type[FileType] = BinaryFile) -> FileType:
+    @overload
+    def as_file(self) -> BinaryFile:
+        ...
+
+    @overload
+    def as_file(self, file_type: Type[FileType]) -> FileType:
+        ...
+
+    def as_file(self, file_type: Type[File[Any, Any]] = BinaryFile) -> File[Any, Any]:
         """Retrieve the content of the fragment as a file."""
         if self.file:
             return file_type(self.file.ensure_serialized())
